@@ -4,6 +4,7 @@ import torch.distributed as dist
 from torchdistlog import logging
 import pandas as pd 
 import traceback
+import numpy as np 
 
 from ...core import models
 from ..misc import utils
@@ -35,7 +36,8 @@ class BaseManager:
         primary_metric=["test", "recall_at_1"],
         to_device_list=["models", "collectors"],
         to_wrap_list=["models"],
-        patience=10,
+        patience=15,
+        early_stop_thres=0.6,
     ):
         self.trainer = trainer
         self.tester = tester
@@ -56,6 +58,7 @@ class BaseManager:
         self.to_device_list = to_device_list
         self.to_wrap_list = to_wrap_list
         self.patience = patience
+        self.early_stop_thres = early_stop_thres
 
         self.best_metric = -1
         self.patience_counts = 0
@@ -167,83 +170,102 @@ class BaseManager:
         is_to_device = "models" in self.to_device_list
         is_to_wrap = "models" in self.to_wrap_list
         if is_to_device:
-            self._members_to_device("models", to_warp=is_to_wrap)
+            for k, v in self.models.items():
+                is_to_wrap = (
+                    "models/{}".format(k) in self.to_wrap_list or
+                    "models" in self.to_wrap_list
+                )
+                self.models[k] = self._members_to_device(v, to_wrap=is_to_wrap)
     
     def initiate_collectors(self):
         # to device
         is_to_device = "collectors" in self.to_device_list
         is_to_wrap = "collectors" in self.to_wrap_list
         if is_to_device:
-            self._members_to_device("collectors", to_warp=is_to_wrap)
+            for k, v in self.collectors.items():
+                self.collectors[k] = self._members_to_device(v, to_wrap=is_to_wrap)
     
     def initiate_selectors(self):
         # to device
         is_to_device = "selectors" in self.to_device_list
         is_to_wrap = "selectors" in self.to_wrap_list
         if is_to_device:
-            self._members_to_device("selectors", to_warp=is_to_wrap)
+            for k, v in self.selectors.items():
+                self.selectors[k] = self._members_to_device(v, to_wrap=is_to_wrap)
 
     def initiate_losses(self):
         # to device
         is_to_device = "losses" in self.to_device_list
         is_to_wrap = "losses" in self.to_wrap_list
         if is_to_device:
-            self._members_to_device("losses", to_warp=is_to_wrap)
+            for k, v in self.losses.items():
+                self.losses[k] = self._members_to_device(v, to_wrap=is_to_wrap)
     
     def initiate_schedulers(self):
         if self.schedulers is None:
             self.schedulers = {}
     
-    def _members_to_device(self, module_name: str, to_warp=True):
-        members = getattr(self, module_name)
+    def _members_to_device(self, module, to_wrap=True):
         # to device
         if not self.is_distributed:
             # single-device
             if self.multi_gpu:
-                for k, v in members.items():
-                    members[k] = members[k].to(self.device)
-                    if to_warp:
-                        if self.device_wrapper_type == "DP":
-                            members[k] = torch.nn.DataParallel(
-                                v,
-                                device_ids=self.device_ids
+                module = module.to(self.device)
+                if to_wrap:
+                    if self.device_wrapper_type == "DP":
+                        module = torch.nn.DataParallel(
+                            module,
+                            device_ids=self.device_ids
+                        )
+                    else:
+                        try:
+                            module = DDP(
+                                module,
+                                device_ids=self.device_ids,
+                                find_unused_parameters=True
                             )
-                        else:
-                            try:
-                                members[k] = DDP(
-                                    v,
-                                    device_ids=self.device_ids,
-                                    find_unused_parameters=True
-                                )
-                            except:
-                                trace = traceback.format_exc()
-                                logging.warning("{}".format(trace))
+                        except:
+                            trace = traceback.format_exc()
+                            logging.warning("{}".format(trace))
             else:
-                for k, v in members.items():
-                    members[k] = v.to(self.device)
+                module = module.to(self.device)
         else:
             # multi-device
-            for k, v in members.items():
-                members[k] = members[k].to(self.device)
-                try:
-                    members[k] = DDP(
-                        members[k], 
-                        device_ids=self.device_ids,
-                        find_unused_parameters=True
-                    )
-                except:
-                    trace = traceback.format_exc()
-                    logging.warning("{}".format(trace))
+            module = module.to(self.device)
+            try:
+                module = DDP(
+                    module, 
+                    device_ids=self.device_ids,
+                    find_unused_parameters=True
+                )
+            except:
+                trace = traceback.format_exc()
+                logging.warning("{}".format(trace))
+        return module
     
     """
     Run
     """
     
-    def run(self, phase="train", start_epoch=0, total_epochs=61, is_test=True, is_save=True, interval=1, warm_up=2, warm_up_list=None):
+    def run(self, phase="train", start_epoch=0, total_epochs=61, is_test=True, is_save=True, interval=1, to_test_epoch=[0], warm_up=2, warm_up_list=None):
         self.phase = phase
+        self.interval = interval if interval > 1 else 1
+        self.to_test_epoch = to_test_epoch
         self.assert_phase()
         self.prepare()
         self.maybe_resume(is_save=is_save)
+        def to_test(epoch):
+            if epoch < np.max(self.to_test_epoch):
+                if epoch in self.to_test_epoch:
+                    return True
+                else:
+                    return False
+            else:
+                if epoch % self.interval == 0:
+                    return True
+                else:
+                    return False
+
         if self.phase == "train":
             for _ in range(start_epoch, total_epochs):
                 # train phase
@@ -257,26 +279,27 @@ class BaseManager:
                 self.release_memory()
 
                 # test phase
-                # to_test = True
-                # if self.is_distributed:
-                #     if dist.get_rank() != 0:
-                #         to_test = False
-                # if to_test:
                 if is_test:
-                    if (self.epochs % interval) == 0:
+                    if to_test(self.epochs):
                         self.metrics = self.tester.test()
-                        self.display_metrics()
+                        self.check_if_best()
                         self.save_metrics()
                         self.release_memory()
                 if is_save:
                     self.save_models()
-                
-                # early stop
-                if self.patience_counts >= self.patience:
-                    logging.info("Training terminated!")
-                    break
+                if is_test:
+                    if to_test(self.epochs):
+                        self.display_metrics()
+                        # early stop
+                        if self.patience_counts >= self.patience:
+                            logging.info("Training terminated!")
+                            break
+                        if self.cur_metric <= self.early_stop_thres * self.best_metric:
+                            logging.info("Setting collapsed!")
+                            break
         elif self.phase == "evaluate":
             self.metrics = self.tester.test()
+            self.check_if_best()
             self.display_metrics()
     
     def prepare(self):
@@ -314,6 +337,7 @@ class BaseManager:
         self.epochs = -1
         self.test()
         self.save_metrics()
+        self.check_if_best()
         self.display_metrics()
     
     def save_metrics(self):
@@ -321,19 +345,20 @@ class BaseManager:
             data, _ = self.recorder.get_data({k:v})
             self.recorder.update(data, self.epochs)
     
-    def display_metrics(self):
+    def check_if_best(self):
         # best metric check
-        cur_metric = self.metrics[self.primary_metric[0]][self.primary_metric[1]]
-        if cur_metric > self.best_metric:
-            self.best_metric = cur_metric
+        self.cur_metric = self.metrics[self.primary_metric[0]][self.primary_metric[1]]
+        if self.cur_metric > self.best_metric:
+            self.best_metric = self.cur_metric
             self.is_best = True
             logging.info("NEW BEST METRIC!!!")
             self.patience_counts = 0
         else:
             self.is_best = False
-            self.patience_counts += 1
+            self.patience_counts += self.interval
         self.metrics[self.primary_metric[0]]["BEST_" + self.primary_metric[1]] = self.best_metric
 
+    def display_metrics(self):
         # display
         for k, v in self.metrics.items():
             logging.info("{} Metrics ---".format(k.upper()))
